data "google_client_config" "default" {}

provider "kubernetes" {
  host                   = "https://${google_container_cluster.default.endpoint}"
  token                  = data.google_client_config.default.access_token
  cluster_ca_certificate = base64decode(google_container_cluster.default.master_auth[0].cluster_ca_certificate)

  ignore_annotations = [
    "^autopilot\\.gke\\.io\\/.*",
    "^cloud\\.google\\.com\\/.*"
  ]
}

resource "kubernetes_deployment_v1" "default" {
  metadata {
    name = "devops24-webservice-deployment"
  }

  spec {
    replicas = 3

    selector {
      match_labels = {
        app = "devops24-webservice"
      }
    }

    template {
      metadata {
        labels = {
          app = "devops24-webservice"
        }
      }

      spec {
        container {
          image = "registry.bht-berlin.de:443/masi9606/webservice:0.0.36"
          name  = "devops24-webservice-container"

          port {
            container_port = 8080
            name           = "devops24-svc"
          }

          env {
            name  = "HOST"
            value = "0.0.0.0"
          }

          env {
            name  = "PORT"
            value = "8080"
          }

          security_context {
            allow_privilege_escalation = false
            privileged                 = false
            read_only_root_filesystem  = false

            capabilities {
              add  = []
              drop = ["NET_RAW"]
            }
          }

          liveness_probe {
            http_get {
              path = "/health"
              port = "devops24-svc"

              http_header {
                name  = "X-Custom-Header"
                value = "Awesome"
              }
            }

            initial_delay_seconds = 3
            period_seconds        = 3
          }
        }

        security_context {
          run_as_non_root = true

          seccomp_profile {
            type = "RuntimeDefault"
          }
        }

        # Toleration is currently required to prevent perpetual diff:
        # https://github.com/hashicorp/terraform-provider-kubernetes/pull/2380
        toleration {
          effect   = "NoSchedule"
          key      = "kubernetes.io/arch"
          operator = "Equal"
          value    = "amd64"
        }
      }
    }
  }
}

resource "kubernetes_service_v1" "default" {
  metadata {
    name = "devops24-webservice-loadbalancer"
    annotations = {
      "cloud.google.com/l4-rbs" = "enabled"
    }
  }

  spec {
    selector = {
      app = "devops24-webservice"
    }

    ip_family_policy = "RequireDualStack"

    port {
      name       = "http"
      port       = 80
      target_port = "devops24-svc"
      protocol   = "TCP"
    }

    type = "LoadBalancer"
  }

  depends_on = [time_sleep.wait_service_cleanup]
}

resource "kubernetes_ingress_v1" "default" {
  metadata {
    name = "devops24-webservice-ingress"
    annotations = {
      "kubernetes.io/ingress.class" = "nginx"
    }
  }

  spec {
    rule {
      http {
        path {
          path      = "/"
          path_type = "Prefix"

          backend {
            service {
              name = kubernetes_service_v1.default.metadata[0].name
              port {
                number = 80
              }
            }
          }
        }
      }
    }
  }
}

# Provide time for Service cleanup
resource "time_sleep" "wait_service_cleanup" {
  depends_on = [google_container_cluster.default]

  destroy_duration = "180s"
}
